/*
---------------------------------------------------------------------
 This file is part of CPB (Correlated Pattern Biclustering Algorithm)
 Copyright (c) 2011, 
 By:    
		Onur Kucuktunc,
        Mehmet Deveci,        
        Kemal Eren,
		Doruk Bozdag, 
        Umit V. Catalyurek
---------------------------------------------------------------------
 For license info, please see the README.txt and LICENSE.txt files in the main directory.
---------------------------------------------------------------------
 * Correlation score calculation for a dataset and found biclusters
 *
 *  Created on: Oct 5, 2011
 *      Author: onur kucuktunc
 */

#include <stdio.h>
#include <string.h>
#include <string>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <iostream>
#include <sstream>
#include <fstream>
#include <vector>
#include <algorithm>
using namespace std;

#define ELEMTYPE float

// arguments and variables
const char* input_file;
int numrows = -1;
int numcols = -1;
int rr = -1;
int numbic = -1;
float* corrscores;
struct bicluster {
	vector<unsigned int> I;
	vector<unsigned int> J;
};
vector<bicluster> bics;


/*
 * Read the command line arguments and check if the parameters are correct
 */
void readArguments(int argc, const char * argv[]) {
	if (argc != 6) {
		printf("Usage: corrscore <inputfile> <#rows> <#cols> <reference row> <#biclusters>\n\n");
		exit(0);
	}
	int i = 0;
	i++; assert (i < argc);
	input_file = argv[i];
	i++; assert (i < argc);
	numrows = atoi(argv[i]);
	i++; assert (i < argc);
	numcols = atoi(argv[i]);
	i++; assert (i < argc);
	rr = atoi(argv[i]);
	i++; assert (i < argc);
	numbic = atoi(argv[i]);
	bool argumentsvalid = input_file!=NULL && numrows>0 && numcols>0 &&
			rr>=0 && numbic>=0;
	if(!argumentsvalid) {
		printf("Usage: corrscore <inputfile> <#rows> <#cols> <reference row> <#biclusters>\n\n");
		exit(0);
	}
	cout << "numbic: " << numbic << ", rows/cols: " << numrows << "/" << numcols << endl;
}

/*
 * Read the input file and generate gene expression dataset
 */
void readBiclusters() {
	std::fstream input(input_file, ios::in);
	string csvLine;
	std::string csvElement;
	unsigned int num;
	// read every line from the stream
	for (int i=0; i<numbic; i++)
	{
		if (!std::getline(input, csvLine)) {
			cout << "Error reading biclusters." << endl;
			exit(1);
		}
		// read rows
		vector<unsigned int> I;
		std::istringstream rowStream(csvLine);
		while (std::getline(rowStream, csvElement, ' ')) {
			num = atoi(csvElement.c_str());
//			cout << "read " << num << endl;
			I.push_back(num);
		}

		// read columns
		vector<unsigned int> J;
		std::getline(input, csvLine);
		std::istringstream colStream(csvLine);
		while (std::getline(colStream, csvElement, ' ')) {
			num = atoi(csvElement.c_str());
			J.push_back(num);
		}

		// calculate easy corr score
		vector<unsigned int>::iterator xi;
		for (xi=I.begin(); xi != I.end(); xi++) {
			corrscores[*xi] += J.size();
		}

		// empty line
		std::getline(input, csvLine);

		// create bicluster
		bicluster C;
		C.I = I;
		C.J = J;

		bics.push_back(C);
	}
}


/*
 * Check if this bicluster in already in results
 */
bool existsInResults(vector<unsigned int> &I, vector<unsigned int> &J) {
	unsigned int isize = I.size(), jsize = J.size();

	if (bics.empty()) return false;

	bool found = false;
	vector<bicluster>::iterator vit;
	vector<unsigned int>::iterator i1, i2;
	for (vit=bics.begin(); vit<bics.end(); vit++) {
		if (vit->I.size()!=isize || vit->J.size()!=jsize) continue;
		found = true;
		for (i1=I.begin(), i2=vit->I.begin(); found && i1<I.end(); i1++, i2++)
			if (*i1!=*i2)
				found = false;
		for (i1=J.begin(), i2=vit->J.begin(); found && i1<J.end(); i1++, i2++)
			if (*i1!=*i2)
				found = false;
		if (found)
			return true;
	}
	return false;
}

void outputSet(vector<unsigned int> &bic) {
	vector<unsigned int>::iterator i;
	for (i=bic.begin() ; i != bic.end(); i++) {
		cout << *i << " ";
	}
	cout << endl;
}

void outputBiclusters(vector<bicluster> &bicss) {
	vector<bicluster>::iterator vit;
	vector<unsigned int>::iterator i1, i2;
	for (vit=bicss.begin(); vit<bicss.end(); vit++) {
		outputSet(vit->I);
		outputSet(vit->J);
		cout << endl;
		/*
		for (i1=vit->I.begin() ; i1 != vit->I.end(); i1++) {
			cout << *i1 << " ";
		}
		cout << endl;
		for (i2=vit->J.begin() ; i2 != vit->J.end(); i2++) {
			cout << *i2 << " ";
		}
		cout << endl << endl;
		*/
	}
}

/**
 * Find the number of biclusters that include all the given rows and columns
 */
int IRICsetcount(int r1, int r2, int c1, int c2) {
//	vector<bicluster> iric;
	int count = 0;

//	bool allfound;
	vector<unsigned int>::iterator i1, i2;
	vector<bicluster>::iterator vit;
	for (vit=bics.begin(); vit<bics.end(); vit++) {
		if (find(vit->I.begin(), vit->I.end(), r1) == vit->I.end())
			continue;
		if (find(vit->I.begin(), vit->I.end(), r2) == vit->I.end())
			continue;
		if (find(vit->J.begin(), vit->J.end(), c1) == vit->J.end())
			continue;
		if (find(vit->J.begin(), vit->J.end(), c2) == vit->J.end())
			continue;
		count++;
		/*
		allfound = true;
		for (i1=rows.begin() ; i1 != rows.end(); i1++)
			if (find(vit->I.begin(), vit->I.end(), *i1) == vit->I.end()) {
				allfound = false;
				break;
			}

		if (!allfound) continue;

		for (i2=cols.begin() ; i2 != cols.end(); i2++)
			if (find(vit->J.begin(), vit->J.end(), *i2) == vit->J.end()) {
				allfound = false;
				break;
			}

		if (allfound)
			iric.push_back(*vit);
		*/
	}

//	outputBiclusters(iric);
//	return iric.size();
	return count;
}

/**
 * Find a set of biclusters that include the given row and column
 */
vector<bicluster> IRICsingle(int r, int c) {
	vector<bicluster> iric;

//	bool allfound;
	vector<bicluster>::iterator vit;
	for (vit=bics.begin(); vit<bics.end(); vit++) {
		if (find(vit->I.begin(), vit->I.end(), r) == vit->I.end())
			continue;
		if (find(vit->J.begin(), vit->J.end(), c) == vit->J.end())
			continue;
		iric.push_back(*vit);
	}

//	outputBiclusters(iric);
	return iric;
}

/**
 * Calculate the cell uniqueness score for the given bicluster and cell
 */
float cellUniqueness(bicluster &b, int r, int c) {
	float cu = 0;

	vector<unsigned int> rows;
	rows.push_back(r);
	vector<unsigned int> cols;
	cols.push_back(c);

	vector<unsigned int>::iterator xi, yk;

	for (xi=b.I.begin(); xi != b.I.end(); xi++) {
		if (*xi==(unsigned int)rr) continue;
		for (yk=b.J.begin(); yk != b.J.end(); yk++) {
			/*rows.push_back(*xi);
			cols.push_back(*yk);*/
//			cout << rows.size() << "+" << cols.size() << endl;
			cu += 1.0/IRICsetcount(r,*xi,c,*yk);
			/*rows.pop_back();
			cols.pop_back();*/
		}
	}

	cu /= (b.I.size()-1) * b.J.size();
//	cout << "CU("<<r<<","<<c<<")=" << cu << endl;
	return cu;
}

/**
 * Calculate the overlap score for the given cell
 */
float overlapScore(int r, int c) {
	float os = 0;
	vector<bicluster> Bv = IRICsingle(r, c);

	vector<bicluster>::iterator it;
	for (it=Bv.begin() ; it != Bv.end(); it++)
		os += cellUniqueness(*it, r, c);

	return os;
}

/**
 * Calculate the correlation score of the given row (gene)
 */
float correlationScore(int r) {
	float cs = 0;
	for (int c=0; c<numcols; c++) {
		cs += overlapScore(r,c);
//		cout << "r:" << r << ", c:" << c << ", cs:" << cs << endl;
	}
	return cs;
}

void computeScores() {

	for (int r=0; r<numrows; r++) {
		corrscores[r] = correlationScore(r);
		cout << corrscores[r] << endl;
	}
}

int main(int argc, const char * argv[]) {

	readArguments(argc, argv);
	corrscores = new float[numrows];
	readBiclusters();

	for (int i=0; i<numrows; i++)
		cout << corrscores[i]/corrscores[rr] << endl;
//	computeScores();

	return 0;
}
