/*
---------------------------------------------------------------------
 This file is part of CPB (Correlated Pattern Biclustering Algorithm)
 Copyright (c) 2011, 
 By:    Doruk Bozdag, 
        Umit V. Catalyurek,
        Mehmet Deveci,
        Onur Kucuktunc,
        Kemal Eren
---------------------------------------------------------------------
 For license info, please see the README.txt and LICENSE.txt files in the main directory.
---------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <float.h>
#include "../util/priorityq.h"

#define CHANGE_KEY 0
#define INSERT_NODE 1
#define DELETE_NODE -1

#define MAXLINE 10000000
#define CDINF 1000

#define MAX(a,b) ((a)>=(b)?(a):(b))
#define MIN(a,b) ((a)<=(b)?(a):(b))

typedef struct {
    int fixed;
    double beta;
    double alpha;
    double r;
} Row_Struct;

typedef struct {
    int fixed;
    double x;
} Column_Struct;

typedef struct 
{ 
    int idx; 
    double val; 
} Sort_Struct; 

int str2ia(char *str, int *intArray, int intArraySize);
int str2da(char *str, double *dblArray, int dblArraySize);
int read_dbl_matrix(char *fname, int *nrow, int *ncol, double ***pM, int header);

int read_fix_info(char *fname, Row_Struct *row, Column_Struct *col, int nrow, int ncol,
                  int *nfixrow, int *nfixcol);
void Improve_Fitness(int cid, int *introw, int *nintrow, int *intcol, int *nintcol,
                     Row_Struct *row, Column_Struct *col, double **M, int nrow, int ncol,
                     Sort_Struct *row_errs, Sort_Struct *col_errs, int min_ncol,
                     double r2cR, double target_r, int nfixrow, int nfixcol, double fixw);
void Converge_Cluster (int nintcol, int *intcol, int *rnintrow, int *introw,
                       Row_Struct *row, Column_Struct *col, double **M,
                       int max_a, double max_slope_change, Sort_Struct *row_errs, double rm_ratio, int nfixrow, double fixw);
double Least_Squares_PCC (int i, double **M, int nintcol, int *intcol, Row_Struct *row, Column_Struct *col);
void Compute_Error (double **M, int nrow, int ncol, int *introw, int nintrow, int *intcol, int nintcol,
                    Row_Struct *row, Column_Struct *col, Sort_Struct *row_err, Sort_Struct *col_err,
                    int turn, int nfixrow, double fixw);
void Sort_wrt_vals(Sort_Struct *errs, int nelts);
    
#ifdef _DEBUG2
void print_row_info(int *introw, int nintrow, Row_Struct *row, char *comment);
void print_col_info(int *intcol, int nintcol, Column_Struct *col, char *comment);
void print_col_errors(int nelts, Sort_Struct *errs, char *comment);
void print_row_errors(int nelts, Sort_Struct *errs, char *comment, Row_Struct *row);
#endif
