/*
---------------------------------------------------------------------
 This file is part of CPB (Correlated Pattern Biclustering Algorithm)
 Copyright (c) 2011, 
 By:    Doruk Bozdag, 
        Umit V. Catalyurek,
        Mehmet Deveci,
        Onur Kucuktunc,
        Kemal Eren
---------------------------------------------------------------------
 For license info, please see the README.txt and LICENSE.txt files in the main directory.
---------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

/*######################################################################################################*/
/* CREATE INITIAL BICLUSTERS */
int main (int argc, char *argv[])
{
    int nrow, ncol, nclus, minnrow, maxnrow, fixedrow, fixedcol;
    int *selected, nselrow, rowno, i, j;
    FILE *fp;

    if (argc < 9) {
        printf("%s: Create initial biclusters, potentialy with fixed rows and columns\n", argv[0]);
        printf("Args: <num rows> <num columns> <num biclusters> <min rows> <max rows> <fixed row | -1> <fixed column | -1> <output filename>\n");
        exit(1);
    }
    nrow = atoi(argv[1]);
    ncol = atoi(argv[2]);
    nclus = atoi(argv[3]);
    minnrow = atoi(argv[4]);
    maxnrow = atoi(argv[5]);
    if (maxnrow > nrow) {
        fprintf(stderr, "max row should be greater than num rows");
        exit(1);
    }
    fixedrow = atoi(argv[6]);
    fixedcol = atoi(argv[7]);
    if (!(fp = fopen(argv[8], "w"))) {
        fprintf(stderr, "Could not open output file\n");
        exit(1);
    }
    
    srand(time(NULL));

    selected = (int *) malloc(sizeof(int) * nrow);
    memset(selected, 0xff, sizeof(int) * nrow);   


    for (i=0; i<nclus; i++) {
        /* Randomly select nselrow rows */  
        nselrow = minnrow + rand() % (maxnrow-minnrow+1);

        /* Print cluster id in the first line */
        fprintf(fp, "BICLUSTER %d\n", i);
        
        /* Print the fixed row in the second line */
        fprintf(fp, "%d\n", fixedrow);
        if (fixedrow != -1) {
            nselrow--;
            selected[fixedrow] = i;
        } 
        
        /* Print the selected rows in the third line */
        for (j=0; j<nselrow; j++) {
            do {
                rowno = rand() % nrow;
            } while (selected[rowno] == i);
            selected[rowno] = i;
            fprintf(fp, "%d ", rowno);
        }
        fprintf(fp, "\n");
        
        /* Print the fixed columns in the fourth line */
        fprintf(fp, "%d\n", fixedcol);

        /* Print the selected columns in the fifth line */
        for (j=0; j<ncol; j++) 
            if (j != fixedcol)
                fprintf(fp, "%d ", j);
        fprintf(fp, "\n");
    }
    fclose(fp);
    
    free(selected);
    return 0;
}
