/*
---------------------------------------------------------------------
 This file is part of CPB (Correlated Pattern Biclustering Algorithm)
 Copyright (c) 2011, 
 By:    Doruk Bozdag, 
        Umit V. Catalyurek,
        Mehmet Deveci,
        Onur Kucuktunc,
        Kemal Eren
---------------------------------------------------------------------
 For license info, please see the README.txt and LICENSE.txt files in the main directory.
---------------------------------------------------------------------
*/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#define _str2arr_c

/***************************************************************************************/
/** Convert string of integers into the integer array.*/
/** Return number of integers. */
/** Adequate space should have been allocated for intArray. Otherwise exit with error. */
int str2ia(char *str, int *intArray, int intArraySize)
{
    char *nptr = str, *endptr = NULL;
    int n = 0;

    for ( ; !isdigit(*nptr) && *nptr != '\n' && *nptr != '-'; nptr++);
    while (isdigit(*nptr) || *nptr == '-') {
        intArray[n++] = (int) strtol(nptr, &endptr, 10);        
        nptr = endptr;
        for ( ; !isdigit(*nptr) && *nptr != '\n' && *nptr != '-'; nptr++);
        if (n >= intArraySize) {
            printf ("\n Error in str2ia: Please increase the size of intArray. n: %d, intArraySize: %d\n", n, intArraySize);
            exit(1);
        }
    }
    return n;
}

/***************************************************************************************/
/** Convert string of doubles into the double array.*/
/** Return number of doubles. */
/** Adequate space should have been allocated for dblArray. Otherwise exit with error. */
int str2da(char *str, double *dblArray, int dblArraySize)
{
    char *nptr = str, *endptr = NULL;
    int n = 0;

    for ( ; !isdigit(*nptr) && *nptr != '\n' && *nptr != '-'; nptr++);
    while (isdigit(*nptr) || *nptr == '-') {        
        dblArray[n++] = strtod(nptr, &endptr);
        nptr = endptr;
        for ( ; !isdigit(*nptr) && *nptr != '\n' && *nptr != '-'; nptr++);
        if (n >= dblArraySize) {
            printf ("\n Error in str2da: Please increase the size of dblArray. n: %d, dblArraySize: %d\n", n, dblArraySize);
            exit(1);
        }
    }
    return n;
}
