###################################################################################################
###---------------------------------------------------------------------------------------------###
### This file is part of CPB (Correlated Pattern Biclustering Algorithm)                        ###
### Copyright (c) 2011,                                                                         ###
### By:    Mehmet Deveci,                                                                       ###
###        Onur Kucuktunc,                                                                      ###
###        Kemal Eren,                                                                          ###
###        Doruk Bozdag,                                                                        ###
###        Umit V. Catalyurek                                                                   ###
###                                                                                             ###
###---------------------------------------------------------------------------------------------###
### For license info, please see the README.txt and LICENSE.txt files in the main directory.    ###
###---------------------------------------------------------------------------------------------###
###################################################################################################


import os
import sys
import numpy
import math 

startCol = 2

def getRowCol(soft):
    """
    Returns the number of rows, number of columns as well as the names of the probes and names of the genes in the dataset.
    @param soft: A microarray dataset. Assumed to have 1 line of column labels, and 2 columns of probes and gene names. (startCol, and startRow parameters)
    """
    probeGenes = []
    f = open(soft,"r")
    a = f.readline()
    cols = len(a.split("\t")) - startCol 
    i = 0
    for line in f:
        rr = line.split("\t")
        probeGene = [rr[0], rr[1]]
        probeGenes.append(probeGene)
        i+=1
    f.close()
    return i, cols, probeGenes
    
def getNumBic(found):
    """
    Returns the number of found biclusters.
    @param found: path to the file where found biclusters are saved in the following format;
	    <rows>                                          \n
	    <cols>                                          \n
        seperated by empty lines
    """
    f = open(found, "r")
    i = 0
    for line in f:
        i += 1
    f.close()
    return i/3

def getArray(tmp_out):
    """
    Returns the array of the correlation score of each gene which is the output of "correlation"
    @param tmp_out: path to the output of "correlation".
    """
    f = open(tmp_out)
    arr = []
    f.readline()
    for line in f:
        val = float(line)
        if(math.isnan(val)):
            arr.append(0)
        else:
            arr.append(float(line))
    f.close()
    return numpy.array(arr)

def print_usage():
    print "python run_correlation.py <reference_row> <input_folder> <output_file>"
    print "\treference_row : is the integer index of reference row where the first row is 0th row."
    print "\tinput_folder : from which the datasets and their results will be read. See TODO: http://bmi.osu.edu/hpc/software.html for the example of format of the folder"
    print "\toutput_file: where the output file should be written."

try:
    refrence = int(sys.argv[1])
    input_folder = sys.argv[2]
    outfile = sys.argv[3]
except Exception:
    print_usage()
    sys.exit()

tmpOut = outfile


arr = []
run = 0
for item in os.listdir(input_folder):
    abs = os.path.join(os.path.abspath(input_folder), item)
    if os.path.isdir(abs):
        found = os.path.join(abs, str(refrence) +"_found.txt")
        soft = os.path.join(abs, item + ".soft")

        row, column, probeGenes = getRowCol(soft)
        num_bic = getNumBic(found)
        
        runStr = ("correlation " + found + " " + str(row) + " " + str(column) + " " + str(refrence) + " " + str(num_bic))
        os.system( runStr +" > " + tmpOut)
        if run == 0:
            arr = getArray(tmpOut)
        else:   
            arr += getArray(tmpOut)
        run += 1




dtype = [('id', int), ('sc', float)]
v = []
ind = 0
for line in arr:
    v.append((ind, line))
    ind += 1

v = numpy.array(v, dtype=dtype)
v = numpy.sort(v,order='sc')
f = open(outfile,"w")
tot = len(v)
for i in range(len(v)):
    ind = tot - i - 1
    myInd = v[ind][0]
    f.write(str(probeGenes[myInd][0]) + "\t" + probeGenes[myInd][1] + "\t" + str(myInd) + "\t" + str(v[ind][1]) + "\n")

f.close()





        
        
    
