###################################################################################################
###---------------------------------------------------------------------------------------------###
### This file is part of CPB (Correlated Pattern Biclustering Algorithm)                        ###
### Copyright (c) 2011,                                                                         ###
### By:    Mehmet Deveci,                                                                       ###
###        Onur Kucuktunc,                                                                      ###
###        Kemal Eren,                                                                          ###
###        Doruk Bozdag,                                                                        ###
###        Umit V. Catalyurek                                                                   ###
###                                                                                             ###
###---------------------------------------------------------------------------------------------###
### For license info, please see the README.txt and LICENSE.txt files in the main directory.    ###
###---------------------------------------------------------------------------------------------###
###################################################################################################

"""
Functions to run a biclustering algorithm on shuffled data matrix. 
"""
import numpy

def random_max(data, alg):
    """
    Function runs algorithm on shuffled version of the given data.
    Returns the max bicluster with the highest number of rows. 
    @param data : 2 dimensional data matrix.
    @param alg : function to run biclustering algorithm. alg takes single parameter which is the data matrix.
    """
    shuffled = shuffleMatrix(data)
    found = alg(shuffled)
    return get_max_row_bicluster(found)
    
def get_max_row_bicluster(found):
    """
    Returns the bicluster with maximum number of rows among the given list.
    @param found : A list of biclusters.
    """
    max = found[0]
    for bic in found:
        if len(max.rows) < len(bic.rows):
            max = bic
    return max

def shuffleMatrix(data):
    """
    Function to shuffle the elements of the given data matrix.
    @param data: 2 dimensional data matrix.
    """
    data = numpy.array(data)
    oneDim = data.reshape(data.shape[0]* data.shape[1],1)
    numpy.random.shuffle(oneDim)
    twoDim = oneDim.reshape(data.shape[0], data.shape[1])
    return twoDim
    
