/*
---------------------------------------------------------------------
 This file is part of CPB (Correlated Pattern Biclustering Algorithm)
 Copyright (c) 2011, 
 By:    Doruk Bozdag, 
        Umit V. Catalyurek,
        Mehmet Deveci,
        Onur Kucuktunc,
        Kemal Eren
---------------------------------------------------------------------
 For license info, please see the README.txt and LICENSE.txt files in the main directory.
---------------------------------------------------------------------
*/

#include <stdlib.h>
#include <stdio.h>

#ifndef MAXLINE
#define MAXLINE 10000000
#endif

int str2da(char *str, double *dblArray, int dblArraySize);
    
/*#####################################################################*/
/* READ A MATRIX INTO DOUBLE** M. M IS ALLOCATED INSIDE THIS FNCN #####*/
int read_dbl_matrix(char *fname, int *nrow, int *ncol, double ***pM, int header)
{
    FILE *fp;
    char *line;
    double *dblline;
    int i, j, readcol;
    double **M;
    
    line = (char *) malloc(MAXLINE * sizeof(char));
    dblline = (double *) malloc((MAXLINE/2+1) * sizeof(int));

    fp = fopen(fname, "r");
    if (!fp) {
        printf("Error: Could not open matrix file %s\n", fname);
        exit(1);
    }
    
    if (header) {
        fgets(line, MAXLINE-1, fp);   
        if (str2da(line, dblline, MAXLINE/2+1) != 2) {
            printf("The first line of the matrix being read is not right. Required format: <Number of rows> <Number of columns>\n");
            exit(-1);
        }
        *nrow = (int) dblline[0];
        *ncol = (int) dblline[1];
    } else {        
        for (i=0; fgets(line, MAXLINE-1, fp); i++) {
            if (i == 0) { 
                *ncol = str2da(line, dblline, (MAXLINE/2+1)); /* Determine number of columns */
                continue;
            }
            if ((readcol = str2da(line, dblline, (MAXLINE/2+1))) != *ncol) {
                printf("The %dth row of the matrix has %d columns, not equal to expected number %d\n", i, readcol, *ncol);
                exit(-1);
            }
        }
        *nrow = i; /* Determine number of rows */
        fclose(fp);
        fp = fopen(fname, "r");
    }

        
    M = *pM;
    M = (double **) malloc(*nrow * sizeof(double));
    for (i=0; i<*nrow; i++) 
        M[i] = (double *) malloc(*ncol * sizeof(double));

    
    for (i=0; i<*nrow && fgets(line, MAXLINE-1, fp); i++) {
        if ((readcol = str2da(line, dblline, (MAXLINE/2+1))) != *ncol) {
            printf("The %dth row of the matrix has %d columns, not equal to expected number %d\n", i, readcol, *ncol);
            exit(-1);
        }
        for (j=0; j<*ncol; j++)        
            M[i][j] = dblline[j];
    }
    if (i < *nrow) {
        printf("Number of rows read is smaller than the expected number %d\n", *nrow);
        exit(-1);
    }

    fclose(fp);
    free(line);
    free(dblline);
    *pM = M;   
    return 0;
}

