/*
---------------------------------------------------------------------
 This file is part of CPB (Correlated Pattern Biclustering Algorithm)
 Copyright (c) 2011, 
 By:    Doruk Bozdag, 
        Umit V. Catalyurek,
        Mehmet Deveci,
        Onur Kucuktunc,
        Kemal Eren
---------------------------------------------------------------------
 For license info, please see the README.txt and LICENSE.txt files in the main directory.
---------------------------------------------------------------------
*/

#include <iostream>
#include <fstream>

using namespace std;

/*### CLASS DEFINITION ##################################*/
template <class T>
class Matrix {
  public:
    Matrix () { ; };
    Matrix (const char *fname);
    ~Matrix();
    void read_file (const char *fname);
    int get_ncol() const { return ncol; }
    int get_nrow() const { return nrow; }
    void get_row(const unsigned int rowid, T *rowcopy) const;
    void print() const;
  private:
    unsigned int nrow;
    unsigned int ncol;
    T **val;
};

/*### CONSTRUCTOR ##################################*/
template <class T>
Matrix<T>::Matrix (const char *fname) {
    read_file(fname);
}

/*### DESTRUCTOR ###################################*/
template <class T>
Matrix<T>::~Matrix() {
    for (unsigned int i=0; i<nrow; i++) 
        delete [] val[i];
    delete [] val;
}

/*### READ_FILE #####################################*/
template <class T>
void Matrix<T>::read_file (const char *fname) {

    /* Open input file */
    ifstream ifs (fname, ios::in);
    if (!ifs) {
        cerr << "Could not open file " << fname << endl;
        exit(1);
    }
    unsigned int i, j;

    /* Read the header line */
    ifs >> i >> j; 
    nrow = i;
    ncol = j;

    /* Allocate matrix */
    val = new T*[nrow];    
    if (!val) {
        cerr << "Allocation error for Matrix.val[][]" << endl;
        exit(1);
    }
    for (i=0; i<nrow; i++) {
        val[i] = new T[ncol];
        if (!val[i]) {
            cerr << "Allocation error for Matrix.val[]" << endl;
            exit(1);
        }        
    }

    /* Read the matrix entries */
    for (i=0; i<nrow; i++) {
        for (j=0; j<ncol && !ifs.eof(); j++) {
            ifs >> val[i][j];
        }
    }
    if (i != nrow || j != ncol) {
        cerr << "Error in reading Matrix. Check the header line" << endl;
        exit(1);
    }
}

/*### GET COPY OF A ROW #####################################*/
template <class T>
void Matrix<T>::get_row(const unsigned int rowid, T *rowcopy) const {
    if (!rowcopy) {
        cout << "Error in Matrix::get_row(rowid, rowcopy). rowcopy should have been allocated\n";
        exit(1);
    }
    for (unsigned int j=0; j<ncol; j++) 
        rowcopy[j] = val[rowid][j]; 
}

/*### PRINT ######################################*/
template <class T>
void Matrix<T>::print() const {
    cout << nrow << " " << ncol << endl;
    for (unsigned int i=0; i<nrow; i++) {
        for (unsigned int j=0; j<ncol; j++) 
            cout << val[i][j] << " ";
        cout << endl;
    }    
}
    

